@@ -6,6 +6,7 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
+Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -40,6 +41,8 @@ main(int argc,
   Imlib_Load_Error err;
   char *filename_im = NULL, *filename_thumb = NULL;
 
+  char *have_extension = NULL;
+
   time_t t;
   struct tm *tm;
 
@@ -50,6 +53,8 @@ main(int argc,
   if (!opt.output_file) {
     opt.output_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot.png");
     opt.thumb_file = gib_estrdup("%Y-%m-%d-%H%M%S_$wx$h_scrot-thumb.png");
+  } else {
+    have_extension = scrot_have_file_extension(opt.output_file);
   }
 
 
@@ -78,7 +83,13 @@ main(int argc,
   imlib_context_set_image(image);
   imlib_image_attach_data_value("quality", NULL, opt.quality, NULL);
 
+  if (!have_extension) {
+    imlib_image_set_format("png");
+  }
+
   filename_im = im_printf(opt.output_file, tm, NULL, NULL, image);
+  scrot_check_if_overwrite_file(&filename_im);
+
   gib_imlib_save_image_with_error_return(image, filename_im, &err);
   if (err)
     gib_eprintf("Saving to file %s failed\n", filename_im);
@@ -123,6 +134,7 @@ main(int argc,
     else
     {
       filename_thumb = im_printf(opt.thumb_file, tm, NULL, NULL, thumbnail);
+      scrot_check_if_overwrite_file(&filename_thumb);
       gib_imlib_save_image_with_error_return(thumbnail, filename_thumb, &err);
       if (err)
         gib_eprintf("Saving thumbnail %s failed\n", filename_thumb);
@@ -157,15 +169,117 @@ scrot_do_delay(void)
   }
 }
 
+char* scrot_have_file_extension(char *filename)
+{
+    char *ext = strrchr(filename, '.');
+    return (ext && (strlen(ext + 1) == 3)) ? ext : NULL;
+}
+
+void scrot_check_if_overwrite_file(char **filename)
+{
+  char *curfile = *filename;
+
+  if (opt.overwrite == 1) return;
+
+  if (access(curfile, F_OK) == -1) return;
+
+  const int max_count = 999;
+  static int count = 0;
+  const char *extension = scrot_have_file_extension(curfile);
+  const size_t slen = strlen(curfile);
+  int nalloc = slen + 4 + 1; // _000 + NUL byte
+  char fmt[5];
+  char *newname = NULL;
+
+  if (extension)
+    nalloc += 4; // .ext
+
+  newname = calloc(nalloc, sizeof(char));
+
+  if (extension)
+    // exclude extension
+    memcpy(newname, curfile, slen - 4);
+  else
+    memcpy(newname, curfile, slen);
+
+  do {
+    snprintf(fmt, 5, "_%03d", count++);
+
+    if (!extension) {
+      strncpy(newname + slen, fmt, 5);
+    } else {
+        strncpy((newname + slen)-4, fmt, 5);
+        strncat(newname, extension, 4);
+    }
+      curfile = newname;
+  } while ((count < max_count) && (access(curfile, F_OK) == 0));
+
+  free(*filename);
+  *filename = newname;
+
+  if (count == max_count) {
+    fprintf(stderr, "scrot can no longer generate new file names.\n"
+                    "The last attempt is %s\n", newname);
+    free(newname);
+    exit(EXIT_FAILURE);
+  }
+}
+
+
+void
+scrot_grab_mouse_pointer(const Imlib_Image image,
+    const int ix_off, const int iy_off)
+{
+  XFixesCursorImage *xcim = XFixesGetCursorImage(disp);
+
+  const int width       = xcim->width;
+  const int height      = xcim->height;
+  const int x           = (xcim->x - xcim->xhot) - ix_off;
+  const int y           = (xcim->y - xcim->yhot) - iy_off;
+  DATA32 *pixels        = NULL;
+
+#ifdef __i386__
+  pixels = xcim->pixels;
+#else
+  DATA32 data[width * height * 4];
+
+  for (size_t i = 0; i < (width * height); i++)
+    ((DATA32*)data)[i] = (DATA32)xcim->pixels[i];
+
+  pixels = data;
+#endif
+
+  Imlib_Image imcursor  = imlib_create_image_using_data(width, height, pixels);
+
+  XFree(xcim);
+
+  if (!imcursor) {
+     fprintf(stderr, "scrot_grab_mouse_pointer: Failed create image using data.");
+     exit(EXIT_FAILURE);
+  }
+
+  imlib_context_set_image(imcursor);
+  imlib_image_set_has_alpha(1);
+  imlib_context_set_image(image);
+  imlib_blend_image_onto_image(imcursor, 0, 0, 0, width, height, x, y, width, height);
+  imlib_context_set_image(imcursor);
+  imlib_free_image();
+}
+
+
 Imlib_Image
 scrot_grab_shot(void)
 {
   Imlib_Image im;
 
   if (! opt.silent) XBell(disp, 0);
+
   im =
     gib_imlib_create_image_from_drawable(root, 0, 0, 0, scr->width,
                                          scr->height, 1);
+  if (opt.pointer == 1)
+    scrot_grab_mouse_pointer(im, 0, 0);
+
   return im;
 }
 
@@ -174,9 +288,20 @@ scrot_exec_app(Imlib_Image image, struct tm *tm,
                char *filename_im, char *filename_thumb)
 {
   char *execstr;
+  int ret;
 
   execstr = im_printf(opt.exec, tm, filename_im, filename_thumb, image);
-  system(execstr);
+
+  errno = 0;
+
+  ret = system(execstr);
+
+  if (ret == -1) {
+    fprintf(stderr, "The child process could not be created: %s\n", strerror(errno));
+  } else if (WEXITSTATUS(ret) == 127) {
+    fprintf(stderr, "scrot could not be executed the command: %s.\n", execstr);
+  }
+
   exit(0);
 }
 
@@ -193,6 +318,8 @@ scrot_grab_focused(void)
   if (!scrot_get_geometry(target, &rx, &ry, &rw, &rh)) return NULL;
   scrot_nice_clip(&rx, &ry, &rw, &rh);
   im = gib_imlib_create_image_from_drawable(root, 0, rx, ry, rw, rh, 1);
+  if (opt.pointer == 1)
+	  scrot_grab_mouse_pointer(im, rx, ry);
   return im;
 }
 
@@ -232,14 +359,27 @@ scrot_sel_and_grab_image(void)
   if ((XGrabPointer
        (disp, root, False,
         ButtonMotionMask | ButtonPressMask | ButtonReleaseMask, GrabModeAsync,
-        GrabModeAsync, root, cursor, CurrentTime) != GrabSuccess))
-    gib_eprintf("couldn't grab pointer:");
+        GrabModeAsync, root, cursor, CurrentTime) != GrabSuccess)) {
+    fprintf(stderr, "couldn't grab pointer:");
+    XFreeCursor(disp, cursor);
+    XFreeCursor(disp, cursor2);
+    XFreeGC(disp, gc);
+    exit(EXIT_FAILURE);
+  }
+
 
   if ((XGrabKeyboard
        (disp, root, False, GrabModeAsync, GrabModeAsync,
-        CurrentTime) != GrabSuccess))
-    gib_eprintf("couldn't grab keyboard:");
+        CurrentTime) != GrabSuccess)) {
+    fprintf(stderr, "couldn't grab keyboard:");
+    XFreeCursor(disp, cursor);
+    XFreeCursor(disp, cursor2);
+    XFreeGC(disp, gc);
+    exit(EXIT_FAILURE);
+  }
 
+  if (opt.freeze == 1)
+      XGrabServer(disp);
 
   while (1) {
     /* handle events here */
@@ -309,8 +449,10 @@ scrot_sel_and_grab_image(void)
     errno = 0;
     count = select(fdsize, &fdset, NULL, NULL, NULL);
     if ((count < 0)
-        && ((errno == ENOMEM) || (errno == EINVAL) || (errno == EBADF)))
-      gib_eprintf("Connection to X display lost");
+        && ((errno == ENOMEM) || (errno == EINVAL) || (errno == EBADF))) {
+      fprintf(stderr, "Connection to X display lost");
+      exit(EXIT_FAILURE);
+    }
   }
   if (rect_w) {
     XDrawRectangle(disp, root, gc, rect_x, rect_y, rect_w, rect_h);
@@ -319,9 +461,12 @@ scrot_sel_and_grab_image(void)
   XUngrabPointer(disp, CurrentTime);
   XUngrabKeyboard(disp, CurrentTime);
   XFreeCursor(disp, cursor);
+  XFreeCursor(disp, cursor2);
   XFreeGC(disp, gc);
   XSync(disp, True);
 
+  if (opt.freeze == 1)
+      XUngrabServer(disp);
 
   if (done < 2) {
     scrot_do_delay();
@@ -346,6 +491,9 @@ scrot_sel_and_grab_image(void)
 
     if (! opt.silent) XBell(disp, 0);
     im = gib_imlib_create_image_from_drawable(root, 0, rx, ry, rw, rh, 1);
+
+    if (opt.pointer == 1)
+       scrot_grab_mouse_pointer(im, rx, ry);
   }
   return im;
 }
@@ -353,9 +501,9 @@ scrot_sel_and_grab_image(void)
 /* clip rectangle nicely */
 void
 scrot_nice_clip(int *rx, 
-		int *ry, 
-		int *rw, 
-		int *rh)
+  int *ry,
+  int *rw,
+  int *rh)
 {
   if (*rx < 0) {
     *rw += *rx;
@@ -385,7 +533,8 @@ scrot_get_geometry(Window target,
 
   /* get windowmanager frame of window */
   if (target != root) {
-    unsigned int d, x;
+    unsigned int d;
+    int x;
     int status;
     
     status = XGetGeometry(disp, target, &root, &x, &x, &d, &d, &d, &d);
@@ -513,7 +662,10 @@ im_printf(char *str, struct tm *tm,
           strcat(ret, buf);
           break;
         case 't':
-          strcat(ret, gib_imlib_image_format(im));
+          tmp = gib_imlib_image_format(im);
+          if (tmp) {
+            strcat(ret, gib_imlib_image_format(im));
+          }
           break;
         case '$':
           strcat(ret, "$");
