@@ -8,6 +8,8 @@ Copyright 2009      George Danchev <danchev@spnet.net>
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
+Copyright 2019      Daniel Lublin <daniel@lublin.se>
+Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -43,15 +45,40 @@ init_parse_options(int argc, char **argv)
    memset(&opt, 0, sizeof(scrotoptions));
 
    opt.quality = 75;
+   opt.overwrite = 0;
 
    /* Parse the cmdline args */
    scrot_parse_option_array(argc, argv);
 }
 
+static int
+parse_option_required_number(char *str)
+{
+   char *end = NULL;
+   int ret = 0;
+   errno = 0;
+
+   ret = strtol(str, &end, 10);
+
+   if (str == end) {
+      fprintf(stderr, "the option is not a number: %s\n", end);
+      exit(EXIT_FAILURE);
+   }
+
+   if (errno) {
+      perror("strtol");
+      exit(EXIT_FAILURE);
+   }
+
+   return ret;
+}
+
+
+
 static void
 scrot_parse_option_array(int argc, char **argv)
 {
-   static char stropts[] = "a:bcd:e:hmq:st:uv+:z";
+   static char stropts[] = "a:ofpbcd:e:hmq:st:uv+:z";
    static struct option lopts[] = {
       /* actions */
       {"help", 0, 0, 'h'},                  /* okay */
@@ -63,6 +90,9 @@ scrot_parse_option_array(int argc, char **argv)
       {"border", 0, 0, 'b'},
       {"multidisp", 0, 0, 'm'},
       {"silent", 0, 0, 'z'},
+      {"pointer", 0, 0, 'p'},
+      {"freeze", 0, 0, 'f'},
+      {"overwrite", 0, 0, 'o'},
       /* toggles */
       {"thumb", 1, 0, 't'},
       {"delay", 1, 0, 'd'},
@@ -92,7 +122,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.border = 1;
            break;
         case 'd':
-           opt.delay = atoi(optarg);
+           opt.delay = parse_option_required_number(optarg);
            break;
         case 'e':
            opt.exec = gib_estrdup(optarg);
@@ -101,7 +131,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.multidisp = 1;
            break;
         case 'q':
-           opt.quality = atoi(optarg);
+           opt.quality = parse_option_required_number(optarg);
            break;
         case 's':
            opt.select = 1;
@@ -110,7 +140,7 @@ scrot_parse_option_array(int argc, char **argv)
            opt.focused = 1;
            break;
         case '+':
-           opt.debug_level = atoi(optarg);
+           opt.debug_level = parse_option_required_number(optarg);
            break;
         case 'c':
            opt.countdown = 1;
@@ -121,9 +151,20 @@ scrot_parse_option_array(int argc, char **argv)
         case 'z':
            opt.silent = 1;
            break;
+        case 'p':
+           opt.pointer = 1;
+           break;
+        case 'f':
+           opt.freeze = 1;
+           break;
+        case 'o':
+           opt.overwrite = 1;
+           break;
         case 'a':
-	  options_parse_autoselect(optarg);
-	   break;
+           options_parse_autoselect(optarg);
+           break;
+        case '?':
+           exit(EXIT_FAILURE);
         default:
            break;
       }
@@ -193,9 +234,9 @@ options_parse_autoselect(char *optarg)
 
    if (strchr(optarg, ',')) /* geometry dimensions must be in format x,y,w,h   */
    {
-     dimensions[i++] = atoi(strtok(optarg, tokdelim));
-     while (tok = strtok(NULL, tokdelim) )
-        dimensions[i++] = atoi(tok);
+     dimensions[i++] = parse_option_required_number(strtok(optarg, tokdelim));
+     while ((tok = strtok(NULL, tokdelim)) )
+        dimensions[i++] = parse_option_required_number(tok);
      opt.autoselect=1;
      opt.autoselect_x=dimensions[0];
      opt.autoselect_y=dimensions[1];
@@ -212,12 +253,12 @@ options_parse_thumbnail(char *optarg)
    if (strchr(optarg, 'x')) /* We want to specify the geometry */
    {
      tok = strtok(optarg, "x");
-     opt.thumb_width = atoi(tok);
+     opt.thumb_width = parse_option_required_number(tok);
      tok = strtok(NULL, "x");
      if (tok)
      {
-       opt.thumb_width = atoi(optarg);
-       opt.thumb_height = atoi(tok);
+       opt.thumb_width = parse_option_required_number(optarg);
+       opt.thumb_height = parse_option_required_number(tok);
 
        if (opt.thumb_width < 0)
          opt.thumb_width = 1;
@@ -232,7 +273,7 @@ options_parse_thumbnail(char *optarg)
    }
    else
    {
-     opt.thumb = atoi(optarg);
+     opt.thumb = parse_option_required_number(optarg);
      if (opt.thumb < 1)
        opt.thumb = 1;
      else if (opt.thumb > 100)
@@ -243,14 +284,14 @@ options_parse_thumbnail(char *optarg)
 void
 show_version(void)
 {
-   printf(PACKAGE " version " VERSION "\n");
+   printf(SCROT_PACKAGE " version " SCROT_VERSION "\n");
    exit(0);
 }
 
 void
 show_mini_usage(void)
 {
-   printf("Usage : " PACKAGE " [OPTIONS]... FILE\nUse " PACKAGE
+   printf("Usage : " SCROT_PACKAGE " [OPTIONS]... FILE\nUse " SCROT_PACKAGE
           " --help for detailed usage information\n");
    exit(0);
 }
@@ -260,10 +301,10 @@ void
 show_usage(void)
 {
    fprintf(stdout,
-           "Usage : " PACKAGE " [OPTIONS]... [FILE]\n"
+           "Usage : " SCROT_PACKAGE " [OPTIONS]... [FILE]\n"
            "  Where FILE is the target file for the screenshot.\n"
            "  If FILE is not specified, a date-stamped file will be dropped in the\n"
-           "  current directory.\n" "  See man " PACKAGE " for more details\n"
+           "  current directory.\n" "  See man " SCROT_PACKAGE " for more details\n"
            "  -h, --help                display this help and exit\n"
            "  -v, --version             output version information and exit\n"
            "  -a, --autoselect          non-interactively choose a rectangle of x,y,w,h\n"
@@ -283,14 +324,18 @@ show_usage(void)
            "  -t, --thumb NUM           generate thumbnail too. NUM is the percentage\n"
            "                            of the original size for the thumbnail to be,\n"
            "                            or the geometry in percent, e.g. 50x60 or 80x20.\n"
-           "  -z, --silent              Prevent beeping\n"		   
+           "  -z, --silent              Prevent beeping\n"
+           "  -p, --pointer             Capture the mouse pointer.\n"
+           "  -f, --freeze              Freeze the screen when the selection is used: --select\n"
+           "  -o, --overwrite           By default " SCROT_PACKAGE " does not overwrite the files, use this option to allow it.\n"
+
            "\n" "  SPECIAL STRINGS\n"
            "  Both the --exec and filename parameters can take format specifiers\n"
-           "  that are expanded by " PACKAGE " when encountered.\n"
+           "  that are expanded by " SCROT_PACKAGE " when encountered.\n"
            "  There are two types of format specifier. Characters preceded by a '%%'\n"
            "  are interpreted by strftime(2). See man strftime for examples.\n"
            "  These options may be used to refer to the current date and time.\n"
-           "  The second kind are internal to " PACKAGE
+           "  The second kind are internal to " SCROT_PACKAGE
            "  and are prefixed by '$'\n"
            "  The following specifiers are recognised:\n"
            "                  $f image path/filename (ignored when used in the filename)\n"
@@ -300,10 +345,10 @@ show_usage(void)
            "                  $p image pixel size\n"
            "                  $w image width\n"
            "                  $h image height\n"
-           "                  $t image format\n"
+           "                  $t image format (ignored when used in the filename)\n"
            "                  $$  prints a literal '$'\n"
            "                  \\n prints a newline (ignored when used in the filename)\n"
-           "  Example:\n" "          " PACKAGE
+           "  Example:\n" "          " SCROT_PACKAGE
            " '%%Y-%%m-%%d_$wx$h_scrot.png' -e 'mv $f ~/images/shots/'\n"
            "          Creates a file called something like 2000-10-30_2560x1024_scrot.png\n"
            "          and moves it to your images directory.\n" "\n"
