@@ -6,6 +6,8 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
 Copyright 2009      James Cameron <quozl@us.netrek.org>
 Copyright 2010      Ibragimov Rinat <ibragimovrinat@mail.ru>
 Copyright 2017      Stoney Sauce <stoneysauce@gmail.com>
+Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
+
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -45,6 +47,9 @@ struct __scrotoptions
    int thumb;
    int thumb_width;
    int thumb_height;
+   int pointer;
+   int freeze;
+   int overwrite;
    char *output_file;
    char *thumb_file;
    char *exec;
