@@ -4,6 +4,7 @@ Copyright 1999-2000 Tom Gilbert <tom@linuxbrit.co.uk,
                                   gilbertt@linuxbrit.co.uk,
                                   scrot_sucks@linuxbrit.co.uk>
 Copyright 2009      James Cameron <quozl@us.netrek.org>
+Copyright 2019      Daniel T. Borelli <danieltborelli@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -36,12 +37,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <X11/keysym.h>
 #include <X11/Xresource.h>
 #include <X11/cursorfont.h>
+#include <X11/extensions/Xfixes.h>
+#include <X11/Xcursor/Xcursor.h>
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <stdint.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -54,7 +58,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <giblib/giblib.h>
 
 
-#include "config.h"
+#include "scrot_config.h"
 #include "structs.h"
 #include "getopt.h"
 #include "debug.h"
@@ -89,6 +93,12 @@ char *im_printf(char *str, struct tm *tm,
 Imlib_Image scrot_grab_shot_multi(void);
 Imlib_Image stalk_image_concat(gib_list *images);
 
+void scrot_grab_mouse_pointer(const Imlib_Image image,
+		const int ix_off, const int iy_off);
+
+void scrot_check_if_overwrite_file(char **filename);
+char *scrot_have_file_extension(char *filename);
+
 /* Imlib stuff */
 extern Display *disp;
 extern Visual *vis;
